use ExtUtils::MakeMaker;
use Env qw(XERCES_LIB XERCES_INCLUDE XERCESCROOT SWIG);
use strict;
use vars qw($INCLUDES);

if (defined $XERCES_LIB || defined $XERCES_INCLUDE) {
  # this is the default, so do nothing
} elsif (defined $XERCESCROOT) {
  # they are instructing us to use the build directories
  $XERCES_INCLUDE = "$XERCESCROOT/include\n";
  $XERCES_LIB = "$XERCESCROOT/lib\n";
} 

my $LIBS = '';
$INCLUDES = '-I. -IHandler';
if (defined $XERCES_LIB) {
  print STDERR "Using XERCES_LIB = $XERCES_LIB";
  $LIBS = "-L$XERCES_LIB ";
}

if (defined $XERCES_INCLUDE) {
  print STDERR "Using XERCES_INCLUDE = $XERCES_INCLUDE";
  $INCLUDES .= " -I$XERCES_INCLUDE";
}

if (!defined $XERCES_LIB && !defined $XERCES_INCLUDE) {
  print STDERR <<EOW;

   WARNING

You have not defined any of the following environment variables:
   XERCESCROOT
   XERCES_LIB
   XERCES_INCLUDE

These instruct me how to locate the Xerces header files, and the
Xerces dynamic library. If they are installed in a standard system
directory, I will located them without those variables.

However, if they have been installed in a non-standard location
(e.g. '/usr/include/xerces'), then I will need help. See the README
for more info.

Proceeding ...
EOW
}

# replace this with an absolute path if needed
my @handler_headers;
chomp(@handler_headers = `find Handler -name "*.swig.hpp"`);
sub MY::postamble {
  chomp(my @header_files = `find Xerces_headers -name "*.hpp"`);
  push(@header_files,@handler_headers);
  local $" = ' ';
<<TERMINUS;
Xerces.C Xerces.pm: Xerces.i @header_files postSource.pl postModule.pl
	\$(SWIG) -I. -ILib -ILib/perl5 \$(INC) -D\$(SWIG_VERSION) -package XML::Xerces -perl5 -c++ -shadow -o Xerces.C Xerces.i
	perl postSource.pl Xerces.C
	perl postModule.pl Xerces.pm
TERMINUS
}

# set up the list of object files to include in Xerces.o
my $OBJS = "Xerces.o ";
my @handler_objs = @handler_headers;
$OBJS .= join(' ', map {s/\.swig\.hpp/\.o/;$_} @handler_headers);

# if we're to use SWIG, we need to know which version is available
my $swig = $SWIG || 'swig';
my ($sv_maj,$sv_min) = qx[$swig -version 2>&1] =~ /(\d+)\.(\d+)/;

# We need to know which version of libxerces we are to use
my $XERCES_MAJOR_VERSION = '1.4';
# this substitution will yield '1_4' from '1.4' to be used by 'LIBS'
my $XERCES_LIB_VERSION = $XERCES_MAJOR_VERSION;
$XERCES_LIB_VERSION =~ s/\./_/;
$LIBS .= " -lxerces-c$XERCES_LIB_VERSION -lpthread";

# give some nice feedback for the user
print STDERR "Using SWIG version: SWIG_$ {sv_maj}_$ {sv_min}\n";
print STDERR "Using Xerces-C version: $XERCES_MAJOR_VERSION\n";

# write the makefile
WriteMakefile(
  'AUTHOR'      => 'The Xerces-P developers',
  'ABSTRACT'    => 'Perl Interface for Xerces XML API',
  'CCFLAGS'     => '-D_REENTRANT -DDEBIAN -DHAS_BOOL -D$(SWIG_VERSION)',
  'PM'          => {
    'Xerces.pm' => '$(INST_LIB)/XML/Xerces.pm',
    'DOMParse/DOMParse.pm' => '$(INST_LIB)/XML/Xerces/DOMParse.pm',
  },
  'INC'         => $INCLUDES,
  'LIBS'        => $LIBS,
  'NAME'        => 'XML::Xerces',
  'OBJECT'      => $OBJS,
#  'OPTIMIZE'    => '-g',
  'DIR'         => ['DOMParse', 'Handler'],
  'VERSION'     => "$XERCES_MAJOR_VERSION.1",
  'macro'       => {
    'SWIG'         => "$swig",
    'SWIG_VERSION' => "SWIG_$ {sv_maj}_$ {sv_min}",
  },
);
