%module Xerces
%{
#include <stdio.h>
#include <strstream.h>
#include <iostream.h>
#include <string.h>
#include <sax/InputSource.hpp>
#include <sax/SAXException.hpp>
#include <sax/SAXParseException.hpp>
#include <sax/Locator.hpp>
#include <sax/HandlerBase.hpp>
#include <sax2/Attributes.hpp>
#include <sax2/ContentHandler.hpp>
#include <sax2/LexicalHandler.hpp>
#include <sax2/DefaultHandler.hpp>
#include <sax2/SAX2XMLReader.hpp>
#include <sax2/XMLReaderFactory.hpp>
#include <framework/URLInputSource.hpp>
#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <parsers/DOMParser.hpp>
#include <parsers/SAXParser.hpp>
#include <parsers/SAX2XMLReaderImpl.hpp>
#include <dom/DOM.hpp>
#include <framework/LocalFileInputSource.hpp>
#include <framework/MemBufInputSource.hpp>
#include <framework/StdInInputSource.hpp>
#include <framework/URLInputSource.hpp>
#include <PerlErrorCallbackHandler.hpp>
#include <PerlDocumentCallbackHandler.hpp>
#include <PerlContentCallbackHandler.hpp>
#include <PerlExceptionHandler.hpp>
%}

/**************/
/*            */
/*  TYPEMAPS  */
/*            */
/**************/


/**************/
/*            */
/* DOMStrings */
/*            */
/**************/

/***************************************************************************/
/*                                                                         */
/* FOR FUNCTIONS TAKING DOMString AS AN ARGUMENT --                        */
/*    NOW YOU CAN JUST SUPPLY A STRING                                     */ 
/* THIS TYPEMAP CONVERTS PERL-STRINGS TO DOMStrings AUTOMATICALLY          */
/*                                                                         */
/***************************************************************************/
%typemap(perl5, in) DOMString * (DOMString temp) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    temp = DOMString(ptr);
    $target = &temp;
#ifdef SWIG_1_1
  } else if(SWIG_GetPtr($source,(void **) &$target,"DOMString")) {
#else
  } else if(SWIG_ConvertPtr($source,(void **) &$target,SWIGTYPE_p_DOMString)) {
#endif
    croak("Type error in argument 2 of $name, Expected perl-string or DOMString.");
    XSRETURN(1);
  }
}

/***********************************************/
/*                                             */
/* COVER ALL THE BASES FOR DOMString ARGUMENTS */
/*                                             */
/***********************************************/
%typemap(perl5, in) const DOMString * = DOMString *;
%typemap(perl5, in)       DOMString & = DOMString *;
%typemap(perl5, in) const DOMString & = DOMString *;


/*************************************************************************/
/*                                                                       */
/* RETURN VALUE OF DOMString IS AUTOMATICALLY CONVERTED TO A PERL-STRING */
/* (WITHOUT A CALL TO transcode IN THE PERL SCRIPT)                      */
/*                                                                       */
/*************************************************************************/
%typemap(perl5, out) DOMString {
  $target = sv_newmortal();
  sv_setpv( (SV*)$target,(char *) $source->transcode() );
  delete $source;
  ++argvi;
}

/***************************************************/
/*                                                 */
/* COVER ALL THE BASES FOR DOMString RETURN VALUES */
/*                                                 */
/***************************************************/
%typemap(perl5, out) const DOMString = DOMString;



/***********/
/*         */
/* XMLCh * */
/*         */
/***********/

/************************************************************************/
/*                                                                      */
/* FOR FUNCTIONS TAKING XMLCh * (I.E AN XMLCh STRING) AS AN ARGUMENT -- */
/* NOW YOU CAN JUST SUPPLY A STRING.  THIS TYPEMAP CONVERTS             */
/* PERL-STRINGS TO XMLCh STRINGS AUTOMATICALLY                          */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* CAVEAT:                                                              */
/* TO CONVERT STRINGS TO XMLCh STRINGS, A TEMPORARY POINTER MUST BE     */
/* CREATED IN THE in TYPEMAP TO POINT TO MEMORY THAT HOLDS THE          */
/* CONVERSION.  THE MEMORY IS DYNAMIC, SO IT MUST BE FREED AFTER THE C  */
/* FUNCTION THAT USES IT IS CALLED.  THIS IS DONE VIA A "freearg"       */
/* TYPEMAP.  IT IS MAJORLY KLUDGY, BUT IT WORKS.                        */
/*                                                                      */
/* ONE PROBLEM:  A FEW XERCES FUNCTIONS TAKE MULTIPLE ARGUMENTS OF      */
/* TYPE XMLCh STRING.  THEREFORE, A DIFFERENT TEMPORARY VARIABLE MUST   */
/* BE USED FOR EACH ONE.  THIS IS ACCOMPLISHED BY MAKING SEPERATE       */
/* in AND freearg TYPEMAPS FOR EACH ARGUMENT OF A GIVEN NAME. ONCE      */
/* AGAIN, THIS IS KLUDGY, BUT IT WORKS.                                 */
/*                                                                      */
/************************************************************************/

/**********************/
/*                    */
/* msgDomain ARGUMENT */
/*                    */
/**********************/
%typemap(perl5, in) XMLCh *msgDomain (XMLCh *temp_msgDomain) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_msgDomain = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/******************************************************/
/*                                                    */
/* DUP in TYPEMAP FOR const XMLCh *msgDomain ARGUMENT */
/*                                                    */
/******************************************************/
%typemap(perl5, in) const XMLCh *msgDomain = XMLCh *msgDomain;

/*************************************************************/
/*                                                           */
/* KLUDGE! CLEAN UP WHAT temp_msgDomain ABOVE IS POINTING TO */
/*                                                           */
/*************************************************************/
%typemap(perl5, freearg) XMLCh *msgDomain {
  delete[] temp_msgDomain;
}

/***********************************************************/
/*                                                         */
/* DUP freearg TYPEMAP FOR const XMLCh *msgDomain ARGUMENT */
/*                                                         */
/***********************************************************/
%typemap(perl5, freearg) const XMLCh *msgDomain = XMLCh *msgDomain;


/**********************/
/*                    */
/* errorText ARGUMENT */
/*                    */
/**********************/
%typemap(perl5, in) XMLCh *errorText (XMLCh *temp_errorText) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_errorText = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/******************************************************/
/*                                                    */
/* DUP in TYPEMAP FOR const XMLCh *errorText ARGUMENT */
/*                                                    */
/******************************************************/
%typemap(perl5, in) const XMLCh *errorText = XMLCh *errorText;

/*************************************************************/
/*                                                           */
/* KLUDGE! CLEAN UP WHAT temp_errorText ABOVE IS POINTING TO */
/*                                                           */
/*************************************************************/
%typemap(perl5, freearg) XMLCh *errorText {
  delete[] temp_errorText;
}

/***********************************************************/
/*                                                         */
/* DUP freearg TYPEMAP FOR const XMLCh *errorText ARGUMENT */
/*                                                         */
/***********************************************************/
%typemap(perl5, freearg) const XMLCh *errorText = XMLCh *errorText;


/*********************/
/*                   */
/* systemId ARGUMENT */
/*                   */
/*********************/
%typemap(perl5, in) XMLCh *systemId (XMLCh *temp_systemId) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_systemId = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/*****************************************************/
/*                                                   */
/* DUP in TYPEMAP FOR const XMLCh *systemId ARGUMENT */
/*                                                   */
/*****************************************************/
%typemap(perl5, in) const XMLCh *systemId = XMLCh *systemId;

/************************************************************/
/*                                                          */
/* KLUDGE! CLEAN UP WHAT temp_systemId ABOVE IS POINTING TO */
/*                                                          */
/************************************************************/
%typemap(perl5, freearg) XMLCh *systemId {
  delete[] temp_systemId;
}

/**********************************************************/
/*                                                        */
/* DUP freearg TYPEMAP FOR const XMLCh *systemId ARGUMENT */
/*                                                        */
/**********************************************************/
%typemap(perl5, freearg) const XMLCh *systemId = XMLCh *systemId;


/*********************/
/*                   */
/* publicId ARGUMENT */
/*                   */
/*********************/
%typemap(perl5, in) XMLCh *publicId (XMLCh *temp_publicId) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_publicId = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/*****************************************************/
/*                                                   */
/* DUP in TYPEMAP FOR const XMLCh *publicId ARGUMENT */
/*                                                   */
/*****************************************************/
%typemap(perl5, in) const XMLCh *publicId = XMLCh *publicId;

/************************************************************/
/*                                                          */
/* KLUDGE! CLEAN UP WHAT temp_publicId ABOVE IS POINTING TO */
/*                                                          */
/************************************************************/
%typemap(perl5, freearg) XMLCh *publicId {
  delete[] temp_publicId;
}

/**********************************************************/
/*                                                        */
/* DUP freearg TYPEMAP FOR const XMLCh *publicId ARGUMENT */
/*                                                        */
/**********************************************************/
%typemap(perl5, freearg) const XMLCh *publicId = XMLCh *publicId;


/***********************/
/*                     */
/* versionStr ARGUMENT */
/*                     */
/***********************/
%typemap(perl5, in) XMLCh *versionStr (XMLCh *temp_versionStr) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_versionStr = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/*******************************************************/
/*                                                     */
/* DUP in TYPEMAP FOR const XMLCh *versionStr ARGUMENT */
/*                                                     */
/*******************************************************/
%typemap(perl5, in) const XMLCh *versionStr = XMLCh *versionStr;

/**************************************************************/
/*                                                            */
/* KLUDGE! CLEAN UP WHAT temp_versionStr ABOVE IS POINTING TO */
/*                                                            */
/**************************************************************/
%typemap(perl5, freearg) XMLCh *versionStr {
  delete[] temp_versionStr;
}

/************************************************************/
/*                                                          */
/* DUP freearg TYPEMAP FOR const XMLCh *versionStr ARGUMENT */
/*                                                          */
/************************************************************/
%typemap(perl5, freearg) const XMLCh *versionStr = XMLCh *versionStr;


/************************/
/*                      */
/* encodingStr ARGUMENT */
/*                      */
/************************/
%typemap(perl5, in) XMLCh *encodingStr (XMLCh *temp_encodingStr) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_encodingStr = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/********************************************************/
/*                                                      */
/* DUP in TYPEMAP FOR const XMLCh *encodingStr ARGUMENT */
/*                                                      */
/********************************************************/
%typemap(perl5, in) const XMLCh *encodingStr = XMLCh *encodingStr;

/***************************************************************/
/*                                                             */
/* KLUDGE! CLEAN UP WHAT temp_encodingStr ABOVE IS POINTING TO */
/*                                                             */
/***************************************************************/
%typemap(perl5, freearg) XMLCh *encodingStr {
  delete[] temp_encodingStr;
}

/*************************************************************/
/*                                                           */
/* DUP freearg TYPEMAP FOR const XMLCh *encodingStr ARGUMENT */
/*                                                           */
/*************************************************************/
%typemap(perl5, freearg) const XMLCh *encodingStr = XMLCh *encodingStr;


/**************************/
/*                        */
/* standaloneStr ARGUMENT */
/*                        */
/**************************/
%typemap(perl5, in) XMLCh *standaloneStr (XMLCh *temp_standaloneStr) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_standaloneStr = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/**********************************************************/
/*                                                        */
/* DUP in TYPEMAP FOR const XMLCh *standaloneStr ARGUMENT */
/*                                                        */
/**********************************************************/
%typemap(perl5, in) const XMLCh *standaloneStr = XMLCh *standaloneStr;

/*****************************************************************/
/*                                                               */
/* KLUDGE! CLEAN UP WHAT temp_standaloneStr ABOVE IS POINTING TO */
/*                                                               */
/*****************************************************************/
%typemap(perl5, freearg) XMLCh *standaloneStr {
  delete[] temp_standaloneStr;
}

/***************************************************************/
/*                                                             */
/* DUP freearg TYPEMAP FOR const XMLCh *standaloneStr ARGUMENT */
/*                                                             */
/***************************************************************/
%typemap(perl5, freearg) const XMLCh *standaloneStr = XMLCh *standaloneStr;


/*************************/
/*                       */
/* actualEncStr ARGUMENT */
/*                       */
/*************************/
%typemap(perl5, in) XMLCh *actualEncStr (XMLCh *temp_actualEncStr) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_actualEncStr = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/*********************************************************/
/*                                                       */
/* DUP in TYPEMAP FOR const XMLCh *actualEncStr ARGUMENT */
/*                                                       */
/*********************************************************/
%typemap(perl5, in) const XMLCh *actualEncStr = XMLCh *actualEncStr;

/****************************************************************/
/*                                                              */
/* KLUDGE! CLEAN UP WHAT temp_actualEncStr ABOVE IS POINTING TO */
/*                                                              */
/****************************************************************/
%typemap(perl5, freearg) XMLCh *actualEncStr {
  delete[] temp_actualEncStr;
}

/**************************************************************/
/*                                                            */
/* DUP freearg TYPEMAP FOR const XMLCh *actualEncStr ARGUMENT */
/*                                                            */
/**************************************************************/
%typemap(perl5, freearg) const XMLCh *actualEncStr = XMLCh *actualEncStr;


/*******************/
/*                 */
/* target ARGUMENT */
/*                 */
/*******************/
%typemap(perl5, in) XMLCh *target (XMLCh *temp_target) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_target = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/***************************************************/
/*                                                 */
/* DUP in TYPEMAP FOR const XMLCh *target ARGUMENT */
/*                                                 */
/***************************************************/
%typemap(perl5, in) const XMLCh *target = XMLCh *target;

/**********************************************************/
/*                                                        */
/* KLUDGE! CLEAN UP WHAT temp_target ABOVE IS POINTING TO */
/*                                                        */
/**********************************************************/
%typemap(perl5, freearg) XMLCh *target {
  delete[] temp_target;
}

/********************************************************/
/*                                                      */
/* DUP freearg TYPEMAP FOR const XMLCh *target ARGUMENT */
/*                                                      */
/********************************************************/
%typemap(perl5, freearg) const XMLCh *target = XMLCh *target;


/*****************/
/*               */
/* data ARGUMENT */
/*               */
/*****************/
%typemap(perl5, in) XMLCh *data (XMLCh *temp_data) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp_data = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/*************************************************/
/*                                               */
/* DUP in TYPEMAP FOR const XMLCh *data ARGUMENT */
/*                                               */
/*************************************************/
%typemap(perl5, in) const XMLCh *data = XMLCh *data;

/********************************************************/
/*                                                      */
/* KLUDGE! CLEAN UP WHAT temp_data ABOVE IS POINTING TO */
/*                                                      */
/********************************************************/
%typemap(perl5, freearg) XMLCh *data {
  delete[] temp_data;
}

/******************************************************/
/*                                                    */
/* DUP freearg TYPEMAP FOR const XMLCh *data ARGUMENT */
/*                                                    */
/******************************************************/
%typemap(perl5, freearg) const XMLCh *data = XMLCh *data;


/********************/
/*                  */
/* GENERIC ARGUMENT */
/*                  */
/********************/
%typemap(perl5, in) XMLCh * (XMLCh *temp) {
  if (  SvPOK( $source )  ) {
    char *ptr = (char *)SvPV( $source, na );
    $target = temp = XMLString::transcode(ptr);
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}

/*****************************************************/
/*                                                   */
/* DUP in TYPEMAP FOR const XMLCh * GENERIC ARGUMENT */
/*                                                   */
/*****************************************************/
%typemap(perl5, in) const XMLCh * = XMLCh *;

/***************************************************/
/*                                                 */
/* KLUDGE! CLEAN UP WHAT temp ABOVE IS POINTING TO */
/*                                                 */
/***************************************************/
%typemap(perl5, freearg) XMLCh * {
  delete[] temp;
}

/**********************************************************/
/*                                                        */
/* DUP freearg TYPEMAP FOR const XMLCh * GENERIC ARGUMENT */
/*                                                        */
/**********************************************************/
%typemap(perl5, freearg) const XMLCh * = XMLCh *;


/*
 * RETURN VALUE OF getMessage METHOD ON A saxException (WHAT A 
 * perlErrorHandler) FUNCTION SUCH AS "warning" RECEIVES AS AN ARGUMENT
 */
%typemap(perl5, out) XMLCh * {
  char *cptr = XMLString::transcode($source);

  $target = sv_newmortal();
  sv_setpv( (SV*)$target, (char *)cptr );
  delete[] cptr;
  ++argvi;
}

%typemap(perl5, out) const XMLCh * = XMLCh *;


/*
 * FOR MemBufInputSource CONSTRUCTOR, ALWAYS ADOPT BUFFER (I.E. MAKE A COPY 
 * OF IT) SINCE IT IS TAKEN FROM PERL, AND WHO KNOWS WHAT WILL HAPPEN TO IT
 * AFTER IT IS GIVEN TO THE CONSTRUCTOR
 */
%typemap(perl5, in) XMLByte *srcDocBytes {
  STRLEN length;
  if (  SvPOK( $source )  ) {
    XMLByte *xmlbytes = (XMLByte *)SvPV( $source, length );
    $target = new XMLByte[length];
    memcpy($target, xmlbytes, length);
#ifdef SWIG_1_1
    _arg1 = length;	// MAJOR KLUDGE -- SHOULDN'T HAVE TO PASS FROM PERL -- CALCULATED AUTOMATICALLY
#else
    arg1 = length;	// MAJOR KLUDGE -- SHOULDN'T HAVE TO PASS FROM PERL -- CALCULATED AUTOMATICALLY
#endif
  } else {
    croak("Type error in argument 2 of $name, Expected perl-string.");
    XSRETURN(1);
  }
}
%typemap(perl5, in) const XMLByte *srcDocBytes = XMLByte *srcDocBytes;

/* PERL SHOULD IGNORE THIS ARGUMENT -- CALCULATE IN KLUDGE ABOVE FOR MemBufInputSource */
%typemap(perl5, ignore) unsigned int byteCount {
}
%typemap(perl5, ignore) const unsigned int byteCount = unsigned int byteCount;

%typemap(perl5, ignore) bool adoptBuffer {
  $target = true;
}
%typemap(perl5, ignore) const bool adoptBuffer = bool adoptBuffer;

/*
 * FOR perlErrorHandler MEMBER FUNCTIONS, SO PERL SCALAR DOESN'T GET WRAPPED 
 * BY SWIG
 */
%typemap(perl5, in) SV * {
  $target = $source;
}

/*******************/
/*                 */
/*  INCLUDE FILES  */
/*                 */
/*******************/
/* NECESSARY SUPPORT FILES (typedefs 'N THINGS) */
%include "Xerces_headers/util/XercesDefs.hpp"
%include "Xerces_headers/util/XMLURL.hpp"
%include "Xerces_headers/util/XMLException.hpp"
%include "Xerces_headers/util/Compilers/GCCDefs.hpp"

/* NEEDED FOR INITIALIZATION AND TERMINATION */
%include "Xerces_headers/util/PlatformUtils.hpp"

/* FOR SAX 1.0 API */
%include "Xerces_headers/sax/SAXException.hpp"
%include "Xerces_headers/sax/SAXParseException.hpp"
%include "Xerces_headers/sax/ErrorHandler.hpp"
%include "Xerces_headers/sax/DTDHandler.hpp"
%include "Xerces_headers/sax/DocumentHandler.hpp"
%include "Xerces_headers/sax/EntityResolver.hpp"
%include "Xerces_headers/sax/AttributeList.hpp"
%include "Xerces_headers/sax/HandlerBase.hpp"
%include "Xerces_headers/sax/Locator.hpp"

/* FOR SAX 2.0 API */
%include "Xerces_headers/sax2/Attributes.hpp"
%include "Xerces_headers/sax2/ContentHandler.hpp"
%include "Xerces_headers/sax2/LexicalHandler.hpp"
%include "Xerces_headers/sax2/DefaultHandler.hpp"
%include "Xerces_headers/sax2/SAX2XMLReader.hpp"
%include "Xerces_headers/sax2/XMLReaderFactory.hpp"

/* FOR ERROR HANDLING (PARSER EXCEPTIONS IN PARTICULAR) */
%include "PerlErrorCallbackHandler.swig.hpp"
%include "PerlDocumentCallbackHandler.swig.hpp"
%include "PerlContentCallbackHandler.swig.hpp"
%include "PerlExceptionHandler.swig.hpp"

/* INCLUDE FILES IN "dom" DIRECTORY, ESSENTIALLY ALL OF THESE ARE NODE TYPES */
%include "Xerces_headers/dom/DOMString.hpp"
%include "Xerces_headers/dom/DOM_Node.hpp"
%include "Xerces_headers/dom/DOM_Attr.hpp"
%include "Xerces_headers/dom/DOM_CharacterData.hpp"
%include "Xerces_headers/dom/DOM_Text.hpp"
%include "Xerces_headers/dom/DOM_CDATASection.hpp"
%include "Xerces_headers/dom/DOM_Comment.hpp"
%include "Xerces_headers/dom/DOM_DOMImplementation.hpp"
%include "Xerces_headers/dom/DOM_Document.hpp"
%include "Xerces_headers/dom/DOM_DocumentFragment.hpp"
%include "Xerces_headers/dom/DOM_DocumentType.hpp"
%include "Xerces_headers/dom/DOM_Element.hpp"
%include "Xerces_headers/dom/DOM_Entity.hpp"
%include "Xerces_headers/dom/DOM_EntityReference.hpp"
%include "Xerces_headers/dom/DOM_NamedNodeMap.hpp"
%include "Xerces_headers/dom/DOM_NodeFilter.hpp"
%include "Xerces_headers/dom/DOM_NodeIterator.hpp"
%include "Xerces_headers/dom/DOM_NodeList.hpp"
%include "Xerces_headers/dom/DOM_Notation.hpp"
%include "Xerces_headers/dom/DOM_ProcessingInstruction.hpp"
%include "Xerces_headers/dom/DOM_Range.hpp"
%include "Xerces_headers/dom/DOM_TreeWalker.hpp"
%include "Xerces_headers/dom/DOM_XMLDecl.hpp"

/* INPUT SOURCES */
%include "Xerces_headers/sax/InputSource.hpp"
%include "Xerces_headers/framework/LocalFileInputSource.hpp"
%include "Xerces_headers/framework/MemBufInputSource.hpp"
%include "Xerces_headers/framework/StdInInputSource.hpp"
%include "Xerces_headers/framework/URLInputSource.hpp"

/* THE PARSER (PRETTY IMPORTANT) */
%include "Xerces_headers/parsers/SAXParser.hpp"
%include "Xerces_headers/parsers/DOMParser.hpp"
