#!/usr/bin/perl

my($progname) = $0 =~ m"\/([^/]*)";

%operator_symbols = (
  'addition'                  => '+',
  'addition_assignment'       => '+=',
  'subtraction'               => '-',
  'subtraction_assignment'    => '-=',
  'multiplication'            => '*',
  'multiplication_assignment' => '*=',
  'division'                  => '/',
  'division_assignment'       => '/=',
  'modulus'                   => '%',
  'modulus_assignment'        => '%=',

  'left_shift'                => '<<',
  'left_shift_assignment'     => '<<=',
  'right_shift'               => '>>',
  'right_shift_assignment'    => '>>=',

  'assignment'                => '=',

  'bitwise_and'               => '&',
  'bitwise_or'                => '|',
  'bitwise_xor'               => '^',

  'less_than'                 => '<',
  'less_than_or_equal_to'     => '<=',
  'greater_than'              => '>',
  'greater_than_or_equal_to'  => '>=',
  'equal_to'                  => '==',
  'not_equal_to'              => '!=',

  'increment'                 => '++',
  'decrement'                 => '--'  
);


my $num = 0;
++$num while -e "$progname.$num.tmp";

for $file (@ARGV) {
  next unless open FILE, $file;

  open TEMP, ">$progname.$num.tmp";

  while(<FILE>) {
    # for some reason I don't want to figure out SWIG puts a bunch of
    # methods directly in the XML::Xerces namespace that don't belong there
    # and are duplicated within their proper classes, so we delete them
    if (/FUNCTION WRAPPERS/) {
      while (<FILE>) {
	next unless /\#\#\#\#\#\#\#\#\#\#\#\#\#/;
	last;
      }
    }
    if (/sub DESTROY/) {
      while (<FILE>) {
	next unless /^\}/;
	$_ = <FILE>;
	last;
      }
    }

    # now we set these aliases correctly
    s/\*XML::Xerces::/*XML::Xercesc::/;

    #######################################################################
    #
    # ADDS IN "OVERLOAD" MODULE STATEMENTS
    #
    if (/\#\#\s+Class\b/) {
	if (@operators) {
	  print TEMP "use Carp;\nuse overload\n";
	  for $key (@operators) {
	    my $operator_symbol = $operator_symbols{$key};
	    print TEMP <<"EOT";
'$operator_symbol' => sub {
carp "operator '$operator_symbol' assumed to be commutative."
if \$_[2];
 \$_[0]->$key(\$_[1])
},
EOT
	  }
	  print TEMP qq(\t'fallback' => 1\n;\n\n);
	  @operators = ();
	}
    }

    for $key (keys %operator_symbols) {
      push @operators, $key if /\b$key\b/;
    }

    #######################################################################
    #
    # MUNG MODULE for DOMString and XMLCh support
    #
    #    CHANGE "$args[0] = tied(%{$args[0]})"
    #    TO     "$args[0] = tied(%{$args[0]}) || $args[0]"
    # split on multiple lines to be readable, using s{}{}x
   s{
      (\$args\[(\d+)\])\s*=\s*tied\(\%\{\$args\[\2\]\}\)
     }{$& || $1}x;

    #   CHANGE "return undef if (!defined($result))"
    #   TO     "return $result unless ref($result) =~ m[XML::Xerces]"
    # split on multiple lines to be readable, using s{}{}x
    s{
      return\s*undef\s*if\s*\(\!defined\(\$result\)\)
     }{return \$result unless ref(\$result) =~ m[XML::Xerces]}x;

    #######################################################################
    #
    # MUNG NAMESPACE to XML::Xerces
    #
    # this is for SWIG 1.1, because -package is broken
   s/([^:])\bXerces/$ {1}XML::Xerces/g;
   s/package\s+(?!XML::Xerces)([^\s;])/package XML::Xerces::$1/;
#
   # split on multiple lines to be readable, using s<><>x
   s<\@ISA\s*=\s*qw\s*\(\s*XML::Xerces\s*(?!XML::Xerces::)(\S+)\s*\)>
    <\@ISA = qw( XML::Xerces XML::Xerces::$1 )>x;
#
   s/\$(?!XML::Xerces)(\w+)::OWNER/\$XML::Xerces::$1::OWNER/;
   s/bless(.*?)"(?!XML::Xerces)(.*?)"/bless$1"XML::Xerces::$2"/;
   s/tie(.*?)"(?!XML::Xerces)(.*?)"/tie$1"XML::Xerces::$2"/;

#     #######################################################################
#     #
#     # Perl API specific changes
#     #
#     #   DOM_NodeList: automatically convert to perl list
#     #      if called in a list context
    if (/^(\s*)\$(XML::Xerces::)?DOM_NodeList::OWNER/) {
	    print TEMP <<'EOT';
    # automatically convert to perl list if called in a list context
    return $result->to_list() if wantarray;
EOT
    }

    #   DOM_NamedNodeMap: automatically convert to perl hash
    #      if called in a list context
    if (/^(\s*)\$(XML::Xerces::)?DOM_NamedNodeMap::OWNER/) {
	    print TEMP <<'EOT';
    # automatically convert to perl hash if called in a list context
    return $result->to_hash() if wantarray;
EOT
    }

    #   DOM_Node: automatically convert to base class
    if (/^(\s*)\$(XML::Xerces::)?DOM_Node::OWNER/) {
	    print TEMP <<'EOT';
    # automatically convert to base class
    $result = $result->actual_cast();
EOT
    }

    ######################################################################
    #
    # Method Overloads

    # don't print out SWIG's default overloaded methods, we'll make our own
    next if /XML::Xerces.*__overload__/;

    if (/XML::Xerces::XMLReaderFactory_createXMLReader/) {
	print TEMP <<'EOT';
sub createXMLReader {
    my @args = @_;
    if (scalar @args == 2) {
      XML::Xercesc::XMLReaderFactory_createXMLReader__overload__1(@args);
    } else {
      XML::Xercesc::XMLReaderFactory_createXMLReader(@args);
    }
}
EOT
      # we don't print out the function
      next;
    }

    ######################################################################
    #
    # Callback registration

    # fix a scoping bug for setErrorHandler. If we don't maintain an
    # internal reference to the error handler perl object it will
    # get destroyed if it goes out of scope. Then if an error occurs
    # perl will dump core
    #
    # look for: *setErrorHandler = *XML::Xercesc::*_setErrorHandler;
    if (/\*XML::Xercesc::(\w+)_setErrorHandler/) {
      my $class = $1;
      print TEMP <<"EOT";
use vars qw(\$__ERROR_HANDLER);
sub setErrorHandler {
  my (\$self,\$handler) = \@_;
  my \$callback = XML::Xerces::PerlErrorCallbackHandler->new();
  \$callback->set_callback_obj(\$handler);
  XML::Xercesc::$ {class}_setErrorHandler(\$self,\$callback);
  \$__ERROR_HANDLER = \$callback;
}
EOT

      if ($class eq 'DOMParser') {
	print TEMP <<'EOT';
# SWIG gets horribly upset if we don't decrement these refcounts
sub DESTROY {
  undef $__ERROR_HANDLER;
  undef $__ENTITY_RESOLVER;
}
EOT
      } elsif ($class eq 'SAXParser') {
	print TEMP <<'EOT';
# SWIG gets horribly upset if we don't decrement these refcounts
sub DESTROY {
  undef $__DOCUMENT_HANDLER;
  undef $__ERROR_HANDLER;
  undef $__ENTITY_RESOLVER;
}
EOT
      } elsif ($class eq 'SAX2XMLReader') {
	print TEMP <<'EOT';
# SWIG gets horribly upset if we don't decrement these refcounts
sub DESTROY {
  undef $__CONTENT_HANDLER;
  undef $__ERROR_HANDLER;
  undef $__ENTITY_RESOLVER;
}
EOT
      } else {
	warn("unexpected class: $class, when creating setErrorHandler");
      }
      # we don't print out the function
      next;
    }

    # this same bug is likely to affect setEntityResolver() as well
    # look for: *setEntityResolver = *XML::Xercesc::*_setEntityResolver;
    if (/\*XML::Xercesc::(\w+)_setEntityResolver/) {
      my $class = $1;
      print TEMP <<"EOT";
use vars qw(\$__ENTITY_RESOLVER);
sub setEntityResolver {
  my (\$self,\$handler) = \@_;
  my \$callback = XML::Xerces::PerlErrorCallbackHandler->new();
  \$callback->set_callback_obj(\$handler);
  XML::Xercesc::$ {class}_setEntityResolver(\$self,\$callback);
  \$__ENTITY_RESOLVER = \$callback;
}
EOT
      # we don't print out the function
      next;
    }
    # look for: *setDocumentHandler = *XML::Xercesc::SAXParser_setDocumentHandler;
    if (/SAXParser_setDocumentHandler/) {
      print TEMP <<'EOT';
use vars qw($__DOCUMENT_HANDLER);
sub setDocumentHandler {
  my ($self,$handler) = @_;
  my $callback = XML::Xerces::PerlDocumentCallbackHandler->new();
  $callback->set_callback_obj($handler);
  XML::Xercesc::SAXParser_setDocumentHandler($self,$callback);
  $__DOCUMENT_HANDLER = $callback;
}
EOT
      # we don't print out the function
      next;
    }
    # this same bug is likely to affect setContentHandler() as well
    # look for: *setContentHandler = *XML::Xercesc::SAX2XMLReader_setContentHandler;
    if (/SAX2XMLReader_setContentHandler/) {
      print TEMP <<'EOT';
use vars qw($__CONTENT_HANDLER);
sub setContentHandler {
  my ($self,$handler) = @_;
  my $callback = XML::Xerces::PerlContentCallbackHandler->new();
  $callback->set_callback_obj($handler);
  XML::Xercesc::SAX2XMLReader_setContentHandler($self,$callback);
  # maintain an internal reference
  $__CONTENT_HANDLER = $callback;
}
EOT
      # we don't print out the function
      next;
    }

    print TEMP;
  }

  close FILE;

  print TEMP <<\TERMINUS;
############# Class : XML::Xerces::PerlContentHandler ##############
package XML::Xerces::PerlContentHandler;
@ISA = qw();
sub new {
  my $class = shift;
  return bless {}, $class;
}

sub start_element {}
sub end_element {}
sub start_prefix_mapping {}
sub end_prefix_mapping {}
sub skipped_entity {}
sub start_document {}
sub end_document {}
sub reset_document {}
sub characters {}
sub processing_instruction {}
sub set_document_locator {}
sub ignorable_whitespace {}


############# Class : XML::Xerces::PerlDocumentHandler ##############
package XML::Xerces::PerlDocumentHandler;
@ISA = qw();
sub new {
  my $class = shift;
  return bless {}, $class;
}

sub start_element {}
sub end_element {}
sub start_document {}
sub end_document {}
sub reset_document {}
sub characters {}
sub processing_instruction {}
sub set_document_locator {}
sub ignorable_whitespace {}


############# Class : XML::Xerces::PerlErrorHandler ##############
package XML::Xerces::PerlErrorHandler;
@ISA = qw();
sub new {
  my $class = shift;
  return bless {}, $class;
}

sub warning {
  print STDERR "WARNING:\n";
  print STDERR "FILE:    ", $_[1]->getSystemId, "\n";
  print STDERR "LINE:    ", $_[1]->getLineNumber, "\n";
  print STDERR "COLUMN:  ", $_[1]->getColumnNumber, "\n";
  print STDERR "MESSAGE: ", $_[1]->getMessage, "\n";
  exit 1;
}

sub error {
  print STDERR "ERROR:\n";
  print STDERR "FILE:    ", $_[1]->getSystemId, "\n";
  print STDERR "LINE:    ", $_[1]->getLineNumber, "\n";
  print STDERR "COLUMN:  ", $_[1]->getColumnNumber, "\n";
  print STDERR "MESSAGE: ", $_[1]->getMessage, "\n";
  exit 1;
}

sub fatal_error {
  print STDERR "FATAL ERROR:\n";
  print STDERR "FILE:    ", $_[1]->getSystemId, "\n";
  print STDERR "LINE:    ", $_[1]->getLineNumber, "\n";
  print STDERR "COLUMN:  ", $_[1]->getColumnNumber, "\n";
  print STDERR "MESSAGE: ", $_[1]->getMessage, "\n";
  exit 1;
}


sub reset_errors {}


package XML::Xerces::DOM_NodeList;
# convert the NodeList to a perl list
sub to_list {
  my $self = shift;
  my @list;
  for (my $i=0;$i<$self->getLength();$i++) {
    push(@list,$self->item($i));
  }
  return @list;
}

package XML::Xerces::DOM_Entity;
sub to_hash {
  my $self = shift;
  if ($self->hasChildNodes) {
    return ($self->getNodeName(),
            $self->getFirstChild->getNodeValue());
  } else {
    return ($self->getNodeName(), '');
  }
}

package XML::Xerces::AttributeList;
sub to_hash {
  my $self = shift;
  my %hash;
  for (my $i=0;$i<$self->getLength();$i++) {
    $hash{$self->getName($i)} = $self->getValue($i)
  }
  return %hash;
}

package XML::Xerces::DOM_NamedNodeMap;
# convert the NamedNodeMap to a perl hash
sub to_hash {
  my $self = shift;
  my @list;
  for (my $i=0;$i<$self->getLength();$i++) {
    my $node = $self->item($i);
    if ($node->getNodeType == $XML::Xerces::DOM_Node::ENTITY_NODE) {
      push(@list, $node->to_hash());
    } else {
      push(@list, $node->getNodeName());
      push(@list,$node->getNodeValue());
    }
  }
  return @list;
}

package XML::Xerces;

{
  my $isa = sub {
    return 0 unless defined $_[0];

    my %package_hash = ($_[0] => 1);
    my @package_array = ($_[0]);
    my $base = $_[1];

    while(my $class = shift @package_array) {
      return 1 if "$class" eq "$base";
      foreach my $inherit (eval "\@$class\::ISA;") {
        unless ($package_hash{$inherit}) {
          $package_hash{$inherit} = 1;
          push @package_array, $inherit;
        }
      }
    }
    return 0;
  };


  my $reinterpret_cast = sub {
    return undef unless ref $_[1];

    bless $_[1], $_[0];

    my($raw_ref, $tied) = ($_[1] =~ m[=(.*)\(]);

    if ($raw_ref eq 'SCALAR')   { $tied = tied ${$_[1]}; }
    elsif ($raw_ref eq 'ARRAY') { $tied = tied @{$_[1]}; }
    elsif ($raw_ref eq 'HASH')  { $tied = tied %{$_[1]}; }

    return $_[1] unless $tied;

    bless $tied, $_[0];
    return $_[1];
  };

  sub actual_cast {
    return undef unless &$isa( ref($_[0]), 'XML::Xerces::DOM_Node' );
    return $_[0] if $_[0]->isNull;

    my $node_type = $_[0]->getNodeType;
    return &$reinterpret_cast('XML::Xerces::DOM_Text', $_[0])                  if $node_type == $XML::Xerces::DOM_Node::TEXT_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_ProcessingInstruction', $_[0]) if $node_type == $XML::Xerces::DOM_Node::PROCESSING_INSTRUCTION_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_Document', $_[0])              if $node_type == $XML::Xerces::DOM_Node::DOCUMENT_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_Element', $_[0])               if $node_type == $XML::Xerces::DOM_Node::ELEMENT_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_EntityReference', $_[0])       if $node_type == $XML::Xerces::DOM_Node::ENTITY_REFERENCE_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_CDATASection', $_[0])          if $node_type == $XML::Xerces::DOM_Node::CDATA_SECTION_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_Comment', $_[0])               if $node_type == $XML::Xerces::DOM_Node::COMMENT_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_DocumentType', $_[0])          if $node_type == $XML::Xerces::DOM_Node::DOCUMENT_TYPE_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_Entity', $_[0])                if $node_type == $XML::Xerces::DOM_Node::ENTITY_NODE;
    return &$reinterpret_cast('XML::Xerces::DOM_XMLDecl', $_[0])               if $node_type == $XML::Xerces::DOM_Node::XML_DECL_NODE;

    return $_[0];
  }
}


sub quote_content {
  my $node_value = $_[0];

  $node_value =~ s/&/&amp;/g;
  $node_value =~ s/</&lt;/g;
  $node_value =~ s/>/&gt;/g;
  $node_value =~ s/"/&quot;/g;
  $node_value =~ s/'/&apos;/g;

  return $node_value;
}

{
  my $in_element = 0;
  my $output;
  my @output_array;

  $output_array[$XML::Xerces::DOM_Node::TEXT_NODE] = sub {
    $output .= quote_content $_[0]->getNodeValue;
  };

  $output_array[$XML::Xerces::DOM_Node::PROCESSING_INSTRUCTION_NODE] = sub {
    $output .= '<?' . $_[0]->getNodeName;
    if ( length(my $str = $_[0]->getNodeValue) ) { $output .= " $str"; }
    $output .= '?>';
  };

  $output_array[$XML::Xerces::DOM_Node::DOCUMENT_NODE] = sub {
    for(my $child = $_[0]->getFirstChild->actual_cast ;
     !$child->isNull ;
     $child = $child->getNextSibling->actual_cast) {
      &{$output_array[$child->getNodeType]}($child);
    }
  };

  $output_array[$XML::Xerces::DOM_Node::ELEMENT_NODE] = sub {
    ++$in_element;
    ELEMENT: {
      my $node_name = $_[0]->getNodeName;
      $output .= "<$node_name";

      my $attributes = $_[0]->getAttributes;
      my $attribute_count = $attributes->getLength;

      for(my $ix = 0 ; $ix < $attribute_count ; ++$ix) {
        $attribute = $attributes->item($ix);
        $output .= ' ' . $attribute->getNodeName . '="' . quote_content($attribute->getNodeValue) . '"';
      }

      my $child = $_[0]->getFirstChild->actual_cast;
      ($output .= '/>'), last ELEMENT if $child->isNull;

      $output .= '>';
      until($child->isNull) {
        &{$output_array[$child->getNodeType]}($child);
        $child = $child->getNextSibling->actual_cast;
      }
      $output .= "</$node_name>";
    }
    --$in_element;
    $output .= "\n" unless $in_element;
  };

  $output_array[$XML::Xerces::DOM_Node::ENTITY_REFERENCE_NODE] = sub {
    for(my $child = $_[0]->getFirstChild->actual_cast ;
     !$child->isNull ;
     $child = $child->getNextSibling->actual_cast) {
      &{$output_array[$child->getNodeType]}($child);
    }
  };

  $output_array[$XML::Xerces::DOM_Node::CDATA_SECTION_NODE] = sub {
    $output .= '<![CDATA[' . $_[0]->getNodeValue . ']]>';
  };

  $output_array[$XML::Xerces::DOM_Node::COMMENT_NODE] = sub {
    $output .= '<!--' . $_[0]->getNodeValue . '-->';
    $output .= "\n" unless $in_element;
  };

  $output_array[$XML::Xerces::DOM_Node::DOCUMENT_TYPE_NODE] = sub {
    $output .= '<!DOCTYPE ' . $_[0]->getNodeName;

    my $id;
    if ($id = $_[0]->getPublicId) {
      $output .= qq[ PUBLIC "$id"];
      if ($id = $_[0]->getSystemId) {
        $output .= qq[ "$id"];
      }
    } elsif ($id = $_[0]->getSystemId) {
      $output .= qq[ SYSTEM "$id"];
    }

    if ($id = $_[0]->getInternalSubset) {
      $output .= " [$id]";
    }

    $output .= ">\n"
  };

  $output_array[$XML::Xerces::DOM_Node::ENTITY_NODE] = sub {
    $output .= '<!ENTITY ' . $_[0]->getNodeName;

    my $id;
    if ($id = $_[0]->getPublicId) { $output .= qq[ PUBLIC "$id"]; }
    if ($id = $_[0]->getSystemId) { $output .= qq[ SYSTEM "$id"]; }
    if ($id = $_[0]->getNotationName) { $output .= qq[ NDATA "$id"]; }

    $output .= '>';
  };

  $output_array[$XML::Xerces::DOM_Node::XML_DECL_NODE] = sub {
    $output .= '<?xml version="' . $_[0]->getVersion . '" encoding="' . $_[0]->getEncoding;
    if (my $id = $_[0]->getStandalone) { $output .= qq[" standalone="$id]; }
    $output .= "\"?>\n\n";
  };

  sub serialize {
    $output = '';
    &{$output_array[ $_[0]->getNodeType ]}($_[0]);
    $output;
  }
}

#
# NOTICE: We are automatically calling XMLPlatformUtils::Initialize()
#   when the module is loaded. Do not call it on your own.
#
#
XML::Xerces::XMLPlatformUtils::Initialize();

1;
TERMINUS

  close TEMP;

  rename "$progname.$num.tmp", $file;
}


